package de.dfki.madm.mlwizard.metafeatures;

import java.util.HashMap;
import java.util.Map;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.tools.container.Pair;

public class AttributeStats {
    public Map<String, Double> frequencies;
    public Map<Pair<String, String>, Double> conditionalFrequencies;
    public double mean, var, dev, kurtosis, skewness;
    public double[] moments;

    public AttributeStats(ExampleSet exampleSet, ExampleSet nominalExampleSet, Attribute attribute) {
        computeStatistics(exampleSet, attribute);
        computeFrequencies(nominalExampleSet, attribute);
    }
    
    protected void computeStatistics(ExampleSet exampleSet, Attribute attribute) {
        mean = dev = var = kurtosis = skewness = 0.0;
        moments = new double[5];
        double n = (double)exampleSet.size();
        if(attribute.isNumerical()) {
            for(Example example:exampleSet) {
                mean += example.getNumericalValue(attribute);
            }
            mean /= n;
            for(Example example:exampleSet) {
                double v = example.getNumericalValue(attribute);
                double d = mean - v;
                for(int i=0; i<moments.length; i++) {
                    moments[i] += Math.pow(d, i);                    
                }
            }
            for(int i=0; i<moments.length; i++) {
                moments[i] /= n;                    
            }
            
            var = moments[2];
            if(var != 0.0) {
                dev = Math.sqrt(var);
                kurtosis = moments[4] / Math.pow(moments[2], 2.0);
                skewness = moments[3] / Math.pow(dev, 3.0);
            }
        }
    }
    
    protected void computeFrequencies(ExampleSet nominalExampleSet, Attribute attribute) {
        // -- we need the nominal attribute --
        attribute = nominalExampleSet.getAttributes().get(attribute.getName());
        
        frequencies = new HashMap<String, Double>();
        conditionalFrequencies = new HashMap<Pair<String,String>, Double>();
        Attribute label = nominalExampleSet.getAttributes().getLabel();
        // -- count the occurrences of each value --
        for(Example example:nominalExampleSet) {
            String value = example.getNominalValue(attribute);
            if(!frequencies.containsKey(value)) {
                frequencies.put(value, 0.0);
            }
            frequencies.put(value, frequencies.get(value) + 1);
            
            String c = example.getNominalValue(label);
            Pair<String, String> pair = new Pair<String, String>(value, c);
            if(!conditionalFrequencies.containsKey(pair)) {
                conditionalFrequencies.put(pair, 0.0);
            }
            conditionalFrequencies.put(pair, conditionalFrequencies.get(pair) + 1);
        }
        double n = nominalExampleSet.size();
        for (String value : frequencies.keySet()) {
            frequencies.put(value, frequencies.get(value) / n);
        }
        for (Pair<String, String> pair : conditionalFrequencies.keySet()) {
            conditionalFrequencies.put(pair, conditionalFrequencies.get(pair) / n);
        }
    }
}
